import {useLocation} from "react-router-dom";
import {useEffect} from "react";

export function setCookie(name: string, value: any, expirationDate: Date, path: string) {
    let expires = expirationDate.toUTCString();
    document.cookie = `${name}=${value};expires=${expires};path=${path};`
}

export function getCookie(name: string) {
    const decodedCookie = decodeURIComponent(document.cookie);
    const splitCookie = decodedCookie.split("; ");

    for (const element of splitCookie) {
        if (element.indexOf(name) === 0) {
            return element.substring(name.length + 1)
        }
    }
}

export function deleteJwtCookie() {
    document.cookie = `Jwt=; expires=Thu, 01 Jan 1970 00:00:00 UTC; path=/;`;
    console.log("deleted cookie")
}

export function checkJwtStatus(setIsLoggedIn: (value: boolean) => void) {
    const location = useLocation();

    return useEffect(() => {
        console.log("check jwt-status")
        const jwtCookie = getCookie("Jwt");
        console.log("Jwt Cookie " + jwtCookie)

        if (!jwtCookie) {
            // Wenn der JWT-Cookie nicht vorhanden ist
            setIsLoggedIn(false);
            console.log("Jwt not there")
            return;
        }

        // Der JWT-Cookie ist vorhanden - Überprüfe die Gültigkeit
        const tokenParts = jwtCookie.split('.');

        if (tokenParts.length !== 3) {
            // Ungültiges JWT-Format (nicht genau 3 Teile)
            setIsLoggedIn(false);
            console.log("Jwt is not valid")
            return;
        }

        const payload = JSON.parse(atob(tokenParts[1]));
        const expirationTime = payload.exp * 1000; //Umwandlung in Millisekunden
        console.log(expirationTime)
        console.log(Date.now())

        // Überprüfe das Ablaufdatum
        if (Date.now() > expirationTime) {
            // JWT ist abgelaufen
            setIsLoggedIn(false);
            console.log("Jwt is expired")
            return;
        }
        console.log("Token found and valid")
        setIsLoggedIn(true);
    }, [location.pathname]);
}
