import "../stylesheets/RegisterAndLogin.css";
import {useState} from "react";
import {deleteJwtCookie, setCookie} from "../jwt/Cookies.ts";
import {jwtDecode} from "jwt-decode";
import {useNavigate} from "react-router-dom";

function Login() {
    const navigate = useNavigate();

    const [email, setEmail] = useState('');
    const [password, setPassword] = useState('');

    const handleSubmit = async (e: { preventDefault: () => void; }) => {
        e.preventDefault(); //verhindert, dass die Seite neu geladen wird, wenn das Formular abgeschickt wird
        await handleLogin();
    };
    const handleLogin = async () => {
        try {
            const res = await fetch("http://localhost:8080/api/v1/auth/authenticate", {
                method: "POST",
                body: JSON.stringify({email: email, password: password}),
                headers: {
                    "Content-Type": "application/json",
                },
            });
            console.log(res)
            if (res.status === 200) { //delete old cookie and set new one with new expiration date
                const {token} = await res.json();
                deleteJwtCookie();
                const decodedToken = jwtDecode(token);
                // @ts-ignore
                const expirationDate = new Date(decodedToken.exp * 1000); //*1000 Anzahl der Sekunden seit 1.1.1970 darstellt - JS arbeitet jedoch normalerweise mit Millisekunden
                setCookie("Jwt", token, expirationDate, "/");

                navigate('/');

            } else {
                const errorMessage = await res.text();
                console.error(errorMessage)
                window.alert(errorMessage);
            }
        } catch
            (e: any) {
            console.log(e.status);
        }
    }

    return (
        <main>
            <div>
                <h2 className="h2">Login</h2>
                <form onSubmit={handleSubmit}>
                    <div className="form-group">
                        <input className="input"
                               type="text"
                               value={email}
                               onChange={(e) => setEmail(e.target.value)}
                               placeholder="Email"
                        />
                    </div>
                    <div className="form-group">
                        <input className="input"
                               type="password"
                               value={password}
                               onChange={(e) => setPassword(e.target.value)}
                               placeholder="Passwort"
                        />
                    </div>
                    <div>
                        <button className="submitButton" type="submit">Anmelden</button>
                    </div>
                </form>
            </div>

        </main>
    );
}

export default Login;