package hdm.mi.growbros.controllers;

import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.service.GardenService;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api/v1/garden")
@RequiredArgsConstructor
public class GardenController {
    private static final Logger log = LoggerFactory.getLogger(GardenService.class);

    private final GardenService gardenService;

    @GetMapping
    public ResponseEntity<List<Plant>> getPlantsInGarden(
            @RequestParam(value = "sort", required = false) String sort,
            @AuthenticationPrincipal User user
    ) {
        return ResponseEntity.ok(gardenService.getUserPlants(user, sort));
    }

    @GetMapping("/count")
    public int getGardenSize(
            @AuthenticationPrincipal User user
    ) {
        return gardenService.countByUser(user);
    }

    @PostMapping("/add/{plantId}")
    public ResponseEntity<Plant> addPlantToGarden(
            @PathVariable Long plantId,
            @AuthenticationPrincipal User user
    ) {
        final Plant plantAdded = gardenService.addPlantToGarden(plantId, user);
        return ResponseEntity.status(201).body(plantAdded);
    }

    @DeleteMapping("/remove/{plantId}")
    public ResponseEntity<Void> removePlantFromGarden(
            @PathVariable Long plantId,
            @AuthenticationPrincipal User user
    ) {
        gardenService.removeFromGarden(plantId, user);
        return ResponseEntity.status(204).build();
    }

    @DeleteMapping("/remove/all")
    public int removeAllPlantsFromGarden(
            @AuthenticationPrincipal User user
    ) {
        return gardenService.clearGarden(user);
    }
    @PostMapping("/setNewWateringFrequency")
    public void setNewWateringService(@RequestParam("wateringFrequency") int wateringFrequency , @RequestBody GardenEntry gardenEntry){
        gardenEntry.setWateringFrequency(wateringFrequency);
    }



}
