package hdm.mi.growbros.controllers;

import hdm.mi.growbros.exceptions.PlantNotFoundException;
import hdm.mi.growbros.models.dto.CustomPageDto;
import hdm.mi.growbros.models.dto.DeletedPlantDto;
import hdm.mi.growbros.models.dto.PlantUpdateRequest;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.search.SearchRequest;
import hdm.mi.growbros.service.PlantsService;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;

@RestController
@CrossOrigin
@RequestMapping("/api/v1/plants")
@RequiredArgsConstructor
public class PlantsController {
    private final PlantsService plantsService;

    @GetMapping
    public CustomPageDto<Plant> getAllPlants(
            @RequestParam(value = "page", defaultValue = "0") int page,
            @RequestParam(value = "pageSize", defaultValue = "25") int pageSize
    ) {
        return plantsService.getPlants(page, pageSize);
    }

    @GetMapping("/{id}")
    public Plant getPlant(@PathVariable long id) {
        return plantsService.findById(id).orElseThrow(() -> new PlantNotFoundException(id));
    }

    @GetMapping("/randomPlants/{numberOfPlants}")
    public Collection<Plant> getRandomPlants(@PathVariable Integer numberOfPlants) {
        return plantsService.getRandomPlants(numberOfPlants);
    }

    @GetMapping("/search")
    public Collection<Plant> getPlantsSearchedFor(@Valid SearchRequest searchRequest) {
        return plantsService.getPlantsSearchedFor(searchRequest);
    }

    @PostMapping
    public void createPlant(@RequestBody @Valid Plant plant) {
        plantsService.createPlant(plant);
    }

    @DeleteMapping("/{id}")
    public DeletedPlantDto deletePlant(@PathVariable long id) {
        return plantsService.deletePlant(id);
    }

    @PatchMapping("/{id}")
    public Plant updatePlant(@PathVariable long id, @RequestBody PlantUpdateRequest updateRequest) {
        return plantsService.updatePlant(id, updateRequest);
    }
}
