package hdm.mi.growbros.controllers;

import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.service.WishListService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;

import java.util.Collection;

@RestController
@CrossOrigin
@RequestMapping("/api/v1/wishlist")
@RequiredArgsConstructor
public class WishListController {

    private final WishListService wishListService;

    @GetMapping
    public Collection<Plant> getAllPlantsInWishList(@RequestParam(value = "sort", required = false) String sort,
                                                    @AuthenticationPrincipal User user) {
        return wishListService.getWishedPlants(user, sort);
    }

    @GetMapping("/count")
    public int getWishListSize(
            @AuthenticationPrincipal User user) {
        return wishListService.countByUser(user);
    }


    @PostMapping("/add/{plantId}")
    public ResponseEntity<Integer> addPlantToWishlist(@PathVariable Long plantId,
                                                      @AuthenticationPrincipal User user) {
        wishListService.addPlantToWishList(plantId, user);
        return ResponseEntity.status(201).build();
    }

    @DeleteMapping("/remove/{plantId}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void removePlantFromWishlist(@PathVariable Long plantId,
                                        @AuthenticationPrincipal User user) {
        wishListService.removeFromWishList(plantId, user);
    }

    @DeleteMapping("/remove/all")
    public int removeAllPlantsFromWishlist(@AuthenticationPrincipal User user) {
        return wishListService.clearWishList(user);
    }
}