package hdm.mi.growbros.repositories;

import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.user.User;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface GardenRepository extends JpaRepository<GardenEntry, Long> {
    List<GardenEntry> findByUser(User user);

    List<GardenEntry> findByUser(User user, Sort sort);

    int countByUser(User user);

    int deleteAllByUser(User user);

    void deleteByIdAndUser(Long entryId, User user);

    int deleteAllByPlant_Id(Long plantId);

    @Query("""
                SELECT ge FROM GardenEntry ge
                WHERE ge.user = :user
                ORDER BY
                    CASE
                        WHEN :week <= ge.plant.harvestWeekStart
                            THEN ABS(:week - ge.plant.harvestWeekStart)
                        ELSE ABS(52 + :week - ge.plant.harvestWeekStart)
                    END,
                    ge.plant.name
                ASC
            """)
    List<GardenEntry> findAllByNearestHarvestWeek(@Param("user") User user, @Param("week") int currentWeek);

    @Query("""
                SELECT ge FROM GardenEntry ge
                WHERE ge.user = :user
                ORDER BY
                    CASE
                        WHEN :week <= ge.plant.plantWeekStart
                            THEN ABS(:week - ge.plant.plantWeekStart)
                        ELSE ABS(52 + :week - ge.plant.plantWeekStart)
                    END,
                    ge.plant.name
                ASC
            """)
    List<GardenEntry> findAllByNearestPlantingWeek(@Param("user") User user, @Param("week") int currentWeek);

    @Query("""
                SELECT ge.id FROM GardenEntry ge
                WHERE ge.user = :user
                AND ge.plant.id = :plantId
            """)
    Long findEntryIdByUserAndPlantId(@Param("user") User user, @Param("plantId") Long plantId);
}
