package hdm.mi.growbros.repositories;

import hdm.mi.growbros.models.WishListEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;


@Repository
public interface WishListRepository extends JpaRepository<WishListEntry, Long> {

    List<WishListEntry> findByUser(User user);

    List<WishListEntry> findByUser(User user, Sort sort);

    int countByUser(User user);

    int deleteAllByUser(User user);

    void deleteByIdAndUser(long entryId, User user);

    @Query("""
                SELECT ge FROM WishListEntry ge
                WHERE ge.user = :user
                ORDER BY
                    CASE
                        WHEN :week <= ge.plant.plantWeekStart
                            THEN ABS(:week - ge.plant.plantWeekStart)
                        ELSE ABS(52 + :week - ge.plant.plantWeekStart)
                    END,
                    ge.plant.name
                ASC
            """)
    List<WishListEntry> findAllByNearestPlantingWeek(@Param("user") User user, @Param("week") int currentWeek);

    @Query("""
                SELECT ge FROM WishListEntry ge
                WHERE ge.user = :user
                AND ge.plant.plantWeekStart <= :week
                AND ge.plant.plantWeekEnd >= :week
            """)
    List<WishListEntry> findByCurrentPlantingWeek(@Param("user") User user, @Param("week") int currentWeek);

    @Query("""
                SELECT ge.plant FROM WishListEntry ge
                WHERE ge.user = :user
                AND ge.plant.id = :plantId
            """)
    Plant findPlantByUserAndPlantId(@Param("user") User user, @Param("plantId") Long plantId);

    @Query("""
                SELECT ge.id FROM WishListEntry ge
                WHERE ge.user = :user
                AND ge.plant.id = :plantId
            """)
    Long findEntryIdByUserAndPlantId(@Param("user") User user, @Param("plantId") Long plantId);

    boolean existsByUserAndPlant(User user, Plant plant);

    int deleteAllByPlant_Id(long id);
}