package hdm.mi.growbros.service;

import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.WishListEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.repositories.GardenRepository;
import hdm.mi.growbros.repositories.WishListRepository;
import org.springframework.stereotype.Service;
import lombok.RequiredArgsConstructor;

import java.time.LocalDate;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.time.temporal.WeekFields;
import java.util.*;

@Service
@RequiredArgsConstructor
public class GrandmaService {


    private final WishListRepository wishListRepository;



    private final GardenService gardenService;


    private  final GardenRepository gardenRepository;




    public String growingTippDesTages(User user) {
        List<Plant> usersPlants = gardenService.getUserPlants(user , null);
        String tipDesTages;
        Random random = new Random();
        int randomNumber = random.nextInt(usersPlants.size());
        tipDesTages = "Tip des Tages für deinen Garten" + usersPlants.get(randomNumber).getGrowingTips();
        return tipDesTages;
    }


    public List<Plant> getPlantsReadyToGrow( User user) {
        LocalDate today = LocalDate.now();

        int weekNumber = today.get(WeekFields.of(Locale.getDefault()).weekOfWeekBasedYear());

        List<Plant> plantsReadyToGrow = new ArrayList<>();

        List<WishListEntry> wishListEntries = wishListRepository.findByCurrentPlantingWeek(user, weekNumber);

        for (WishListEntry entry : wishListEntries) {

            plantsReadyToGrow.add(entry.getPlant());

        }

        return plantsReadyToGrow;
    }

    public List<GardenEntry> getGardenEntriesNeedingWatering(User user)  {
        List<GardenEntry> plantsNeedingWatering = new ArrayList<>();
        List<GardenEntry> plantsInUsersGarden = gardenRepository.findByUser(user);
        if(plantsInUsersGarden.isEmpty()){
            return plantsNeedingWatering;
        }

        for( int index = 1 ; index < plantsInUsersGarden.size();index++){
            if(plantsInUsersGarden.get(index).getWateringFrequency() == 0  ){
                switch (plantsInUsersGarden.get(index).getPlant().getWaterDemand()){
                    case DRY -> {
                        if(getDaysSinceLastWatering(plantsInUsersGarden.get(index).getTimeSinceLastWatering()) > 14) {
                            plantsNeedingWatering.add(plantsInUsersGarden.get(index));
                        }
                    }
                    case WET -> {
                        if(getDaysSinceLastWatering(plantsInUsersGarden.get(index).getTimeSinceLastWatering()) > 7) {
                            plantsNeedingWatering.add(plantsInUsersGarden.get(index));
                        }
                    }
                    case VERY_WET -> {
                        if(getDaysSinceLastWatering(plantsInUsersGarden.get(index).getTimeSinceLastWatering()) > 3) {
                            plantsNeedingWatering.add(plantsInUsersGarden.get(index));
                        }
                    }
                }
            }else if (getDaysSinceLastWatering(plantsInUsersGarden.get(index).getTimeSinceLastWatering()) > plantsNeedingWatering.get(index).getWateringFrequency()){
                plantsNeedingWatering.add(plantsInUsersGarden.get(index));
            }
        }

        return plantsNeedingWatering;


    }


    private long getDaysSinceLastWatering(LocalDate lastWateringDate) {
        if (lastWateringDate != null) {
            LocalDate currentDate = LocalDate.now();
            return ChronoUnit.DAYS.between(lastWateringDate, currentDate);
        }
        return 0;
    }

    public List<GardenEntry> getPlantsReadyToHarvest (User user){


        List<GardenEntry> usersPlants = gardenRepository.findByUser(user);

        List<GardenEntry> plantsReadyToHarvest = new ArrayList<>();

        for(GardenEntry entry : usersPlants){
            int growthDuration = entry.getPlant().getGrowthDuration();
            LocalDate createdAt = entry.getCreatedAt().toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
            LocalDate currentDate = LocalDate.now();
            if(growthDuration != 0 ) {
                if (growthDuration >= ChronoUnit.WEEKS.between(createdAt, currentDate)) {
                    plantsReadyToHarvest.add(entry);
                }
            }

        }
        return plantsReadyToHarvest;

    }
}
