package hdm.mi.growbros.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component
public class FileReader {
    private final Logger log = LoggerFactory.getLogger(FileReader.class);

    public String getResourceFileAsString(String fileName) throws IOException {
        log.info("Trying to read file: '{}'", fileName);
        ClassLoader classLoader = ClassLoader.getSystemClassLoader();
        try (InputStream inputStream = classLoader.getResourceAsStream(fileName)) {
            if (inputStream == null) {
                log.error("Input stream was null, returning null");
                return null;
            }
            try (
                    InputStreamReader isr = new InputStreamReader(inputStream);
                    BufferedReader reader = new BufferedReader(isr)
            ) {
                Stream<String> lines = reader.lines();
                String collected = lines.collect(Collectors.joining(System.lineSeparator()));
                log.info("Successfully read file.");
                return collected;
            }
        }
    }
}
