package hdm.mi.growbros.util;

import hdm.mi.growbros.models.dto.PlantUpdateRequest;
import hdm.mi.growbros.models.plant.Plant;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class EntityMapperTest {
    private EntityMapper entityMapper;

    @BeforeEach
    void setup() {
        entityMapper = new EntityMapper();
    }

    @Test
    void shouldIgnore_whenEitherArgument_isNull() {
        //arrange
        final Entity entity = getBaseEntity();
        final UpdaterObject updater = new UpdaterObject();

        //act and assert
        assertAll(
                () -> assertDoesNotThrow(() -> entityMapper.map(null, entity)),
                () -> assertDoesNotThrow(() -> entityMapper.map(updater, null))
        );
    }

    @Test
    void shouldUpdate_toNewValues() {
        //arrange
        final Entity entity = getBaseEntity();

        final UpdaterObject updater = new UpdaterObject();
        final String expected = "UPDATED value 1";
        updater.value1 = expected;

        //act
        entityMapper.map(updater, entity);

        //assert
        assertAll(
                () -> assertEquals(1, entity.id),
                () -> assertEquals(expected, entity.value1),
                () -> assertEquals("Value 2", entity.value2)
        );
    }

    @Test
    void shouldWork_onActualModel() {
        //arrange
        String description = "My description";
        final Plant plant = Plant.builder().name("Plant initial name").description(description).build();
        String newName = "new name!";

        final PlantUpdateRequest updateRequest = PlantUpdateRequest.builder().name(newName).build();

        //act
        entityMapper.map(updateRequest, plant);

        //assert
        assertAll(
                () -> assertEquals(newName, plant.getName()),
                () -> assertEquals(description, plant.getDescription())
        );
    }

    private class UpdaterObject {
        private String value1;
        private String value2;
    }

    private class Entity {
        private int id;
        private String value1;
        private String value2;
    }

    private Entity getBaseEntity() {
        final Entity entity = new Entity();
        entity.id = 1;
        entity.value1 = "Value 1";
        entity.value2 = "Value 2";
        return entity;
    }
}
